import pandas as pd
import networkx as nx
import matplotlib.pyplot as plt

def dijkstra(graph, srcid):
    node_list = []            # list of nodes to work on
    dist_to_src = dict()      # distance to source on shortest paths
    prec_node_to_src = dict() # preceding node to destination on shortest paths
    for node in graph.nodes:
        dist_to_src[node] = float('inf')
        prec_node_to_src[node] = None
        node_list.append(node)
    dist_to_src[list(graph.nodes)[srcid]] = 0

    while node_list:
        (curr_node,path_cost) = \
            min([(node,dist_to_src[node]) for node in node_list], key = lambda t: t[1])
        print('(current node,path cost to source) = ', (curr_node,path_cost))
        node_list.remove(curr_node)
        print('node_list = ', node_list)
        
        for neighbor in graph.neighbors(curr_node):
            link_cost = graph.get_edge_data(curr_node, neighbor)[0]['cost']
            alt_cost = dist_to_src[curr_node] + link_cost
            if alt_cost < dist_to_src[neighbor]: 
                # path to the neighbor with shorter dist_to_srcance found
                dist_to_src[neighbor] = alt_cost 
                prec_node_to_src[neighbor] = curr_node
                print('dist_to_src = ', dist_to_src)
                print('prec_node_to_src = ', prec_node_to_src)
    return dist_to_src, prec_node_to_src

def plot_graph(graph):
    plt.subplot(121)
    pos = nx.spring_layout(graph)
    nx.draw(graph, pos, with_labels=True)
    edge_labels=dict([((u,v,),d['cost']) for u,v,d in graph.edges(data=True)])
    nx.draw_networkx_edge_labels(graph, pos, edge_labels=edge_labels, \
            label_pos=0.3, font_size=7)

if __name__=="__main__":
    edgesDf = pd.read_csv('graph_edges.csv')
    nodesDf = pd.read_csv('graph_nodes.csv')
    
    graph = nx.MultiDiGraph()
    
    for i, node in nodesDf.iterrows():
        graph.add_node(node['id'])
    
    for i,edge in edgesDf.iterrows():
        graph.add_edge(edge['src'], edge['dst'], cost=edge['cost'])
    
    nodeName = 'v1'
    srcid = [i for i,name in enumerate(graph.nodes()) if name == nodeName]
    dist_to_src,prec_node_to_src = dijkstra(graph, srcid[0])
    print('dist_to_src = ', dist_to_src)
    print('prec_node_to_src = ', prec_node_to_src)
    
    plot_graph(graph)

    # show graph. this is blocking
    plt.show()
